@tool
class_name HexRegion extends Node3D

@onready var hextile_scene = preload("res://maps/hexbuilder/tiles/hex_tile.tscn")

@export var radius = 8
var tile_qr : Vector2i
@export var tile_xz := Vector2i(0,0):
	set(value):
		tile_xz = value
		_compute_tile_qr()
		
@export var flipped : bool:
	set(value):
		flipped = value
		rotation_degrees.y = 180 if flipped else 0
	
func _ready():
	if get_child_count() == 0:
		create_hex_region()
	_set_position_from_offset()
	call_deferred("_deferred_function")
	
func _deferred_function():
	add_hex_tile_elements()
	
func set_multiplayer_visibility(peer : int, v : bool) -> void:
	$MultiplayerSynchronizer.set_visibility_for(peer, v)

func _compute_tile_qr():
	tile_qr.x = tile_xz.x * (radius * 2 + 1) + tile_xz.y * (radius + 2)
	tile_qr.y = tile_xz.x * -radius + tile_xz.y * (radius - 1)
	if is_inside_tree():
		_set_position_from_offset()

func _set_position_from_offset():
	global_position = _compute_position(tile_qr.x, tile_qr.y)

func _compute_position(q, r):
	var oddq = axial_to_oddq(q, r)
	var i = roundi(oddq.x)
	var j = roundi(oddq.y)
	
	var v = Vector3(0, 0, 0)
	if i % 2 == 0:
		v.x = 3 * i / 2
		v.z = sqrt(3) * j
	else:
		v.x = 1.5 * i
		v.z = sqrt(3)/2 + sqrt(3) * j
	return v * HexTile.mesh_scale
			
func create_hex_region():
	for q in range(-radius, radius+1):
		for r in range(-radius+1, radius):
			if abs(q) > radius or abs(r) > radius or abs(q+r) > radius:
				continue
			var pos = _compute_position(q, r)
			var hextile = hextile_scene.instantiate()
			add_child(hextile, true)
			hextile.owner = self
			hextile.set_global_position(pos)

func axial_to_oddq(q, r):
	var col = q
	var row = r + (q - (q&1)) / 2
	return Vector2(col, row)

func oddq_to_axial(x, z):
	var q = x
	var r = z - (x - (x&1)) / 2
	return Vector2(q, r)

@onready var hex_dense_forest = preload("res://maps/hexbuilder/tiles/hex_dense_forest.tscn")

func add_hex_tile_elements():
	var hextiles = find_children("", "HexTile", false)
	for c in hextiles:
		match c.tile_ground:
			HexTile.TILE_GROUND.FOREST:
				var hextile = hex_dense_forest.instantiate()
				c.add_child(hextile)
			
