@tool
class_name HexTile extends Node3D

const mesh_scale = 3.0

enum TILE_GROUND { PLAIN=1, DESERT=2, FOREST=3, HIGH_GRASS=4, RIVER=5, BUILDING=6, HILL=7 }
@export var tile_ground : TILE_GROUND = TILE_GROUND.PLAIN :
	set(value):
		tile_ground = value
		if Engine.is_editor_hint():
			_init_ground()

func _ready():
	_init_ground()
	# enable fog when not in editor
	if not Engine.is_editor_hint():
		fog_set_visible(true)
	
func _init_ground():
	var color : Color
	match tile_ground:
		TILE_GROUND.PLAIN:
			color = Color.LIGHT_GREEN
		TILE_GROUND.DESERT:
			color = Color.BURLYWOOD
		TILE_GROUND.FOREST:
			color = Color.DARK_GREEN
		TILE_GROUND.HIGH_GRASS:
			color = Color.GREEN_YELLOW
		TILE_GROUND.RIVER:
			color = Color.AQUAMARINE
		TILE_GROUND.BUILDING:
			color = Color.DARK_GOLDENROD
		TILE_GROUND.HILL:
			color = Color.GREEN
		_:
			return
	
	var material = StandardMaterial3D.new()
	material.albedo_color = color
	$Ground.set_surface_override_material(0, material)

func fog_set_visible(v):
	$Fog.set_visible(v)
	
func is_neighbor(node: HexTile):
	return true if global_position.distance_to(node.global_position) < 2 * mesh_scale else false
